/*
	Slimp : Slimp Light and Intelligent Media Player.
	Copyright (C) 2009 Guillaume Royer <groyer.perso@gmail.com>

	This file is a part of Slimp.
	
	Slimp is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	Slimp is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Slimp; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	*/

#include<gtk/gtk.h>
#include<gst/gst.h>
#include<glib.h>
#include<glib/gi18n.h>
#include"slimp.h"
#include"slimp_include.h"

void slimp_playing_volume(GtkScaleButton* button, gdouble value, gpointer data){
  Slimp* slimp;
  slimp = (Slimp*) data;
  GValue volume = {0};
  g_value_init(&volume, G_TYPE_DOUBLE);
  g_value_set_double(&volume, gtk_scale_button_get_value(slimp->player->volume_button));
  if(slimp->playing_state != SLIMP_STATE_NULL) g_object_set_property(G_OBJECT(slimp->playing->pipeline), "volume", &volume);
  return;
}

gboolean slimp_playing_scale(GtkWidget* widget, GdkEventButton* event, gpointer data){
	Slimp* slimp;
	slimp = (Slimp*) data;
	GstState playing_state = GST_STATE_PLAYING;
	
	if(event->type == GDK_BUTTON_PRESS){
		slimp->player->scale_lock = TRUE;
		event->button = 2;
		return FALSE;
	}
	if(event->type == GDK_BUTTON_RELEASE){
		event->button = 2;
		g_signal_emit_by_name (G_OBJECT (widget), "value-changed");
		gst_element_seek (slimp->playing->pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, (gtk_adjustment_get_value(GTK_ADJUSTMENT(slimp->player->adjustment))*1000000000), GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);
		gst_element_get_state(slimp->playing->pipeline, &playing_state, NULL, GST_CLOCK_TIME_NONE); 
		slimp->player->scale_lock = FALSE;
		return FALSE;
	}
}

void slimp_close_stream(Slimp* slimp){
	g_main_loop_quit(slimp->playing->loop);
	slimp->playing_state = SLIMP_STATE_NULL;
	slimp->player->adjustment = gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0); //Create adjustment
	gtk_range_set_adjustment(GTK_RANGE(slimp->player->scale), GTK_ADJUSTMENT(slimp->player->adjustment));
	gtk_label_set_text(GTK_LABEL(slimp->player->time_label), "0:00 / 0:00");
	gtk_label_set_text(GTK_LABEL(slimp->player->button_label), _("Play"));
	gtk_statusbar_pop(GTK_STATUSBAR(slimp->player->status_bar), slimp->player->status_bar_id);
	return;
}

gboolean slimp_playing_bus_call (GstBus *bus, GstMessage *msg, gpointer data){
	Slimp* slimp;
	slimp = (Slimp*) data;
	if(GST_MESSAGE_TYPE(msg)==GST_MESSAGE_EOS){
		g_print(_("End of Stream\n"));
		g_main_loop_quit(slimp->playing->loop);
		slimp->playing_state = SLIMP_STATE_NULL;
		return TRUE;
	}
}

gboolean slimp_playing_print_position(gpointer data){
	GstFormat fmt = GST_FORMAT_TIME;
	Slimp* slimp;
	slimp = (Slimp*) data;
	if(slimp->playing_state != SLIMP_STATE_NULL){
		//Get position	
		gst_element_query_position(GST_ELEMENT(slimp->playing->pipeline), &fmt, &(slimp->playing->position));
		 //Convert in seconds from nanoseconds.
		gtk_label_set_text(GTK_LABEL(slimp->player->time_label), SLIMP_STR_POS_OVER_DUR(slimp->playing->position, slimp->playing->duration));
		if(slimp->player->scale_lock == FALSE){
			gtk_adjustment_set_value(GTK_ADJUSTMENT(slimp->player->adjustment), GST_TIME_AS_SECONDS(slimp->playing->position)); //Refresh adjustment.
		}
	}
	return TRUE;
}

gboolean slimp_playing(GtkWidget *widget, gpointer data){
	Slimp* slimp;
	slimp = (Slimp*) data;

	GstFormat format = GST_FORMAT_TIME;
	GstState playing_state = GST_STATE_PLAYING;
	GstState paused_state = GST_STATE_PAUSED;
	
	if((slimp->playing_state == SLIMP_STATE_NULL) && (slimp->toplay->uri != NULL)){
		slimp->playing_state = SLIMP_STATE_PROCESSING;
		slimp->playing->loop = g_main_loop_new (NULL, FALSE); //Main loop.
	
		//Create pipeline
		slimp->playing->pipeline = gst_element_factory_make("playbin2",	"play_pipeline");
		g_object_set(G_OBJECT(slimp->playing->pipeline), "uri", slimp->toplay->uri, NULL);
                g_object_set (G_OBJECT(slimp->playing->pipeline), "buffer-size", 10000000, NULL);

		//And connect to bus.
		slimp->playing->bus = gst_pipeline_get_bus(GST_PIPELINE(slimp->playing->pipeline));
		gst_bus_add_watch(slimp->playing->bus, slimp_playing_bus_call, slimp); //Pass in data the loop to stop if EOS (End Of Stream).
		gst_object_unref(slimp->playing->bus);

		//Play
		gst_element_set_state(slimp->playing->pipeline, GST_STATE_PAUSED);
		gst_element_get_state(slimp->playing->pipeline, &paused_state, NULL, GST_CLOCK_TIME_NONE); //Wait
		slimp->playing_state = SLIMP_STATE_PAUSED;
		GValue volume = {0};
		g_value_init(&volume, G_TYPE_DOUBLE);
		g_value_set_double(&volume, gtk_scale_button_get_value(slimp->player->volume_button));
		if(slimp->playing_state != SLIMP_STATE_NULL) g_object_set_property(G_OBJECT(slimp->playing->pipeline), "volume", &volume);
		slimp_playing(GTK_WIDGET(slimp->player->button), slimp);

		//Set up the scale
		gst_element_query_duration(GST_ELEMENT(slimp->playing->pipeline), &format, &(slimp->playing->duration));
		slimp->player->adjustment = gtk_adjustment_new(0.0, 0.0, GST_TIME_AS_SECONDS(slimp->playing->duration), 1.0, GST_TIME_AS_SECONDS(slimp->playing->duration), 0.0); //Create adjustment
		gtk_range_set_adjustment(GTK_RANGE(slimp->player->scale), GTK_ADJUSTMENT(slimp->player->adjustment));
		gtk_range_set_update_policy(GTK_RANGE(slimp->player->scale), GTK_UPDATE_DISCONTINUOUS);
		g_signal_connect(G_OBJECT(slimp->player->scale), "button-press-event", G_CALLBACK(slimp_playing_scale), slimp);
		g_signal_connect(G_OBJECT(slimp->player->scale), "button-release-event", G_CALLBACK(slimp_playing_scale), slimp);
		
		//Update status bar
		gtk_statusbar_push(GTK_STATUSBAR(slimp->player->status_bar), slimp->player->status_bar_id, slimp->toplay->uri);
		
		//Set up timecheck
		g_timeout_add(200, (GSourceFunc) slimp_playing_print_position, slimp); 
		g_main_loop_run (slimp->playing->loop);
	
		//After leaving the loop (implies that receive an EOS).	
		gst_element_set_state(slimp->playing->pipeline, GST_STATE_NULL); //Stop the pipeline.
		slimp->playing_state = SLIMP_STATE_NULL;
		return TRUE;
	}
	if(slimp->playing_state == SLIMP_STATE_PLAYING){
		gst_element_set_state(slimp->playing->pipeline, GST_STATE_PAUSED);
		gst_element_get_state(slimp->playing->pipeline, &paused_state, NULL, GST_CLOCK_TIME_NONE); 
		gtk_label_set_text(GTK_LABEL(slimp->player->button_label), _("Play"));
		gtk_widget_show_all(slimp->player->button);
		slimp->playing_state = SLIMP_STATE_PAUSED;
		return TRUE;
	}
	if(slimp->playing_state == SLIMP_STATE_PAUSED){
		gst_element_set_state(slimp->playing->pipeline, GST_STATE_PLAYING);
		gst_element_get_state(slimp->playing->pipeline, &playing_state, NULL, GST_CLOCK_TIME_NONE); 
		gtk_label_set_text(GTK_LABEL(slimp->player->button_label), _("Pause"));
		gtk_widget_show_all(slimp->player->button);
		slimp->playing_state = SLIMP_STATE_PLAYING;
		return TRUE;
	}
}
